library cycle_banner;

import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

class CycleBanner extends StatefulWidget {
  final List _list;
  final IndexedWidgetBuilder _itemBuilder;

  //banner的高度
  final double bannerHeight;

  //普通颜色
  final Color normalColor;

  //指示器选中颜色
  final Color currentColor;

  //指示器大小
  final double indicatorSize;

  //指示器之间的间隔
  final double separatorSize;

  //指示器间隔
  final EdgeInsetsGeometry indicatorMargin;

  //指示器位置
  final AlignmentDirectional indicatorAlignment;

  //是否自动滚动
  final bool isAutoRolling;

  //是否无限滚动
  final bool isCycleRolling;

  //滚动间隔
  final Duration intervalDuration;

  //动画时间
  final Duration animationDuration;

  CycleBanner(this._list, this._itemBuilder, {this.normalColor: Colors.grey, this.currentColor: Colors.blue, this.indicatorSize = 10, this.indicatorMargin = const EdgeInsets.only(bottom: 10), this.indicatorAlignment = AlignmentDirectional.bottomCenter, this.bannerHeight = 200, this.separatorSize = 5, this.isAutoRolling = true, this.isCycleRolling = true, this.intervalDuration = const Duration(milliseconds: 2500), this.animationDuration = const Duration(milliseconds: 500)});

  @override
  _CycleBannerState createState() => _CycleBannerState();
}

class _CycleBannerState extends State<CycleBanner> with WidgetsBindingObserver {
  final int _FAKE_BANNER_SIZE = 100;
  int _DEFAULT_BANNER_SIZE = 5;
  Timer _timer;
  PageController _controller;

  StreamController<int> _streamController;

  @override
  Widget build(BuildContext context) {
    return Container(
        height: widget.bannerHeight,
        child: Stack(
          alignment: widget.indicatorAlignment,
          children: [
            GestureDetector(
              child: PageView.builder(
                controller: _controller,
                itemCount: widget._list == null || widget._list.isEmpty ? 0 : (widget.isCycleRolling ? _FAKE_BANNER_SIZE : widget._list.length),
                itemBuilder: widget._itemBuilder,
                onPageChanged: (index) {
                  if (widget.isCycleRolling) {
                    if (index == 0) {
                      _controller.jumpToPage(_DEFAULT_BANNER_SIZE);
                    } else if (index + 1 == _FAKE_BANNER_SIZE) {
                      _controller.jumpToPage(_DEFAULT_BANNER_SIZE - 1);
                    }
                  }
                  _streamController.add(index);
                },
              ),
              onTapCancel: () {
                changeTouchState(false);
              },
              onTapUp: (TapUpDetails details) {
                changeTouchState(false);
              },
              onTapDown: (TapDownDetails details) {
                changeTouchState(true);
              },
            ),
            StreamBuilder(
                stream: _streamController.stream,
                initialData: _controller.initialPage,
                builder: (BuildContext context, AsyncSnapshot<int> snapshot) {
                  return Container(
                    height: widget.indicatorSize,
                    margin: widget.indicatorMargin,
                    child: ListView.separated(
                      itemCount: widget._list.length,
                      shrinkWrap: true,
                      scrollDirection: Axis.horizontal,
                      separatorBuilder: (BuildContext context, int index) {
                        return SizedBox(width: widget.separatorSize);
                      },
                      itemBuilder: (BuildContext context, int index) {
                        return Container(
                          width: widget.indicatorSize,
                          height: widget.indicatorSize,
                          decoration: BoxDecoration(color: (index == snapshot.data % widget._list.length) ? widget.currentColor : widget.normalColor, borderRadius: BorderRadius.all(Radius.circular(widget.indicatorSize))),
                        );
                      },
                    ),
                  );
                }),
          ],
        ));
  }

  void changeTouchState(bool isTouch) {
    if (!widget.isAutoRolling) return;
    if (isTouch) {
      _timer.cancel();
    } else {
      autoRolling();
    }
  }

  void autoRolling() {
    if (!widget.isAutoRolling) return;
    _timer = new Timer.periodic(widget.intervalDuration, (value) {
      _controller.animateToPage(_controller.page.toInt() + 1, duration: widget.animationDuration, curve: Curves.ease);
    });
  }

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance.addObserver(this);
    _DEFAULT_BANNER_SIZE = widget._list == null || widget._list.length == 0 || !widget.isCycleRolling ? 0 : (_FAKE_BANNER_SIZE / 2).toInt() + _FAKE_BANNER_SIZE % widget._list.length;
    _controller = PageController(initialPage: _DEFAULT_BANNER_SIZE);
    _streamController = StreamController();

    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      autoRolling();
    });
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    switch (state) {
      case AppLifecycleState.inactive:
        break;
      case AppLifecycleState.resumed:
        changeTouchState(false);
        break;
      case AppLifecycleState.paused:
        changeTouchState(true);
        break;
      case AppLifecycleState.detached:
        break;
    }
  }

  @override
  void dispose() {
    if (_timer != null) _timer.cancel();
    if (_controller != null) _controller.dispose();
    if (_streamController != null) _streamController.close();
    WidgetsBinding.instance.removeObserver(this);
    super.dispose();
  }
}
